///--------------------------------------------------------------------------------------
// Scrolling Demo.c
//
// By Vern Jensen. Created in August of 1995
///--------------------------------------------------------------------------------------


#include <SWFPSReport.h>
#include <SWIncludes.h>
#include <SWGameUtils.h>

#include "SWApplication.h"
#include "Scrolling Demo.h"


#define kRequireDiamondsToExit		false		// If true, you must collect all diamonds to exit

#define	kFullScreenWindow			true		// Try me full screen!
#define kWorldRectInset				0			// Make the SpriteWorld smaller?
#define	kInterlacedMode				false		// Turns Interlaced mode on/off
#define kSyncToVBL					false		// Sync SpriteWorld to VBL?
#define kMaxFPS						20			// Set to 0 for unrestricted speed


#define	kSpriteMoveDelta			20			// Try 5, 10, 20, or 40
#define kDiamondSpace				8			// How far apart the diamonds are spaced
												// (try 1!)

#define kSpriteMoveDistance			80			// How far the sprite can move from
												// the center of the screen, in pixels.
												// Try making this value higher!

	// Number of ticks to wait before changing tile image; 0 = change every frame
#define	kExitFrameRate				10
#define kDiamondFrameRate			8			// How often diamond tiles change frames
#define kWallFrameRate				120			// How often wall changes frames

#define kSpritePictResID			202
#define kTilePictResID				200
#define kTileWidth					40
#define kTileHeight					40

#define kStartRow					4			// Starting position of sprite
#define kStartCol					5			// in tile col and row


#define	kLeftArrowKey				0x7B
#define	kRightArrowKey				0x7C
#define	kDownArrowKey				0x7D
#define	kUpArrowKey					0x7E

#define	kLeftKeyPad					0x56
#define	kRightKeyPad				0x58
#define	kDownKeyPad					0x54
#define	kUpKeyPad					0x5B

#define kEscKey						0x35


#define kNoKey						0
#define kLeftKey					1
#define kUpKey						2
#define kRightKey					3
#define kDownKey					4


enum tileIDs
{
	kFirstWallTile,
	kLastWallTile = kFirstWallTile+13,
	kFirstGrassTile,
	kLastGrassTile = kFirstGrassTile+2,
	kBlackTile,
	kDiamondTile,
	kDiamondTile2,
	kLastDiamondTile,
	kFirstExitTile,
	kLastExitTile,
	
	kMaxNumTiles
};



/***********/
/* Globals */
/***********/

SpriteWorldPtr		gSpriteWorldP;
SpriteLayerPtr		gSpriteLayerP;
TileMapStructPtr	gTileMapStructP;
TileMapPtr			gTileMap;
DrawProcPtr			gSpriteDrawProc = SWStdSpriteDrawProc;
SpritePtr			gSimpleSpriteP, gDiamondMeterSpriteP;
WindowPtr			gWindowP;
Rect				gScreenMidRect;
short				gCurrentLevel = 128;
Boolean				gSpriteHitExit = false;

long				gNumDiamondsInMap = 0;	// Number of diamonds in the TileMap
long				gNumDiamonds = 0;		// Number of diamonds sprite has collected

struct moveKeys		// Keeps track of which keys are up and which are down
{
	Boolean	up;
	Boolean	right;
	Boolean	down;
	Boolean	left;
} gKeys;


///--------------------------------------------------------------------------------------
// Main
///--------------------------------------------------------------------------------------

void	main( void )
{
	Initialize(kNumberOfMoreMastersCalls);

	if (SWHasSystem7())
	{
		AllowKeyUpEvents();	// Part of SWGameUtils.c
		SetCursor(*GetCursor(watchCursor));
		
		CreateSpriteWorld();
		SetUpTiling();
		CreateBallSprite();
		CreateDiamondMeterSprite();
		
		SetCursor(&qd.arrow);
		HideCursor();
		
		SetUpAnimation();
		RunAnimation();
		ShutDown();
		
		RestoreEventMask();	// Call this after AllowKeyUpEvents
	}
	else
	{
		CantRunOnThisMachine();
	}
}


///--------------------------------------------------------------------------------------
// CreateSpriteWorld
///--------------------------------------------------------------------------------------

void	CreateSpriteWorld( void )
{
	Rect		offscreenRect, worldRect, windRect;
	RgnHandle	mBarUpdateRgn;
	OSErr		err;
	
	gWindowP = GetNewCWindow(kWindowResID, NULL, (WindowPtr)-1L);
	
	if (gWindowP != NULL)
	{
		if (kFullScreenWindow == true)
		{
			SizeWindow(gWindowP, qd.screenBits.bounds.right, 
					qd.screenBits.bounds.bottom, false);
			MoveWindow(gWindowP, 0, 0, false);
		}
		else
		{
				// Center window in screen
			windRect = gWindowP->portRect;
			CenterRect(&windRect, &qd.screenBits.bounds);
			
				// Make sure window is aligned to long-word boundaries (for 8-bit mode)
			windRect.left = windRect.left>>2<<2;
			
			MoveWindow(gWindowP, windRect.left, windRect.top, false);
		}
		
		ShowWindow(gWindowP);
		SetPort(gWindowP);
		mBarUpdateRgn = SWHideMenuBar(gWindowP); // Must be done *after* showing window!
		EraseRgn(mBarUpdateRgn);
		
		if (kInterlacedMode)
			PaintRect(&gWindowP->portRect);	// Blacken window for Interlaced mode
	}
	else
		CantFindResource();
	
	
	err = SWEnterSpriteWorld();
	FatalError(err);
	
	
	worldRect = gWindowP->portRect;
	InsetRect(&worldRect, kWorldRectInset, kWorldRectInset);
	
	
		// Set size of offscreen area
	offscreenRect = worldRect;
	OffsetRect(&offscreenRect, -offscreenRect.left, -offscreenRect.top);
	

		// Make offscreen area evenly divisible by tile width & height
	if ( (offscreenRect.right/kTileWidth)*kTileWidth != offscreenRect.right)
		offscreenRect.right = (offscreenRect.right/kTileWidth)*kTileWidth + kTileWidth;
	
	if ( (offscreenRect.bottom/kTileHeight)*kTileHeight != offscreenRect.bottom)
		offscreenRect.bottom = (offscreenRect.bottom/kTileHeight)*kTileHeight + kTileHeight;
	
		// Create the scrolling sprite world
	err = SWCreateSpriteWorldFromWindow(&gSpriteWorldP, (CWindowPtr)gWindowP, 
			&worldRect, &offscreenRect, 0);
	FatalError(err);
	
			// Create the sprite layer
	err = SWCreateSpriteLayer(&gSpriteLayerP);
	FatalError(err);
	
		// Add it to the world
	SWAddSpriteLayer(gSpriteWorldP, gSpriteLayerP);
		
	
		// Set the gSpriteDrawProc
	if (gSpriteWorldP->pixelDepth == 8)		// If in 256 colors
	{
		if (kInterlacedMode)
			gSpriteDrawProc = BP8BitInterlacedMaskDrawProc;
		else
			gSpriteDrawProc = BlitPixie8BitMaskDrawProc;
	}
	else if ( !(SW_PPC && gSpriteWorldP->pixelDepth < 8) )		// Not 256 colors
	{
			// Use interlaced drawProcs unless in B&W, where interlacing is ugly
		if (kInterlacedMode && gSpriteWorldP->pixelDepth > 2)
			gSpriteDrawProc = BPAllBitInterlacedMaskDrawProc;
		else
			gSpriteDrawProc = BlitPixieAllBitMaskDrawProc;
	}
}


///--------------------------------------------------------------------------------------
// SetUpTiling
///--------------------------------------------------------------------------------------

void	SetUpTiling( void )
{	
	OSErr		err;

	err = SWInitTiling(gSpriteWorldP, kTileHeight, kTileWidth, kMaxNumTiles);
	FatalError(err);
	
	err = SWLoadTileMap(&gTileMapStructP, gCurrentLevel);
	FatalError(err);
	
	SWInstallTileMap(gSpriteWorldP, gTileMapStructP, 0);
	gTileMap = gTileMapStructP->tileMap;

		// Load the tiles
	err = SWLoadTilesFromPictResource(
		gSpriteWorldP, 
		kFirstWallTile,			// startTileID 
		kLastExitTile,			// endTileID
		kTilePictResID,			// pictResID
		0,						// maskResID
		kNoMask,				// maskType
		0,						// horizBorderWidth
		0);						// vertBorderHeight
	FatalError(err);
	
	CountDiamonds();
}


///--------------------------------------------------------------------------------------
// CountDiamonds - Counts the number of diamonds in the TileMap
///--------------------------------------------------------------------------------------

void	CountDiamonds( void )
{
	short	row, col;
	
	gNumDiamondsInMap = 0;
	gNumDiamonds = 0;
	
	for (row = 0; row < gSpriteWorldP->tileLayerArray[0]->numRows; row++)
	{
		for (col = 0; col < gSpriteWorldP->tileLayerArray[0]->numCols; col++)
		{
			if (gTileMap[row][col] == kDiamondTile)
				gNumDiamondsInMap++;
		}
	}
}

	
///--------------------------------------------------------------------------------------
// CreateBallSprite
///--------------------------------------------------------------------------------------

void	CreateBallSprite( void )
{
	OSErr				err;
	
		// Create the ball sprite
	err = SWCreateSpriteFromCicnResource(gSpriteWorldP, &gSimpleSpriteP, NULL, 
			128, 1, kFatMask);	
	FatalError(err);
	
		// Set up the ball sprite
	SWAddSprite(gSpriteLayerP, gSimpleSpriteP);
	SWSetSpriteMoveProc(gSimpleSpriteP, KeySpriteMoveProc);
	SWSetSpriteLocation(gSimpleSpriteP, kStartCol * kTileWidth, kStartRow * kTileHeight);
	SWSetSpriteMoveDelta(gSimpleSpriteP, 0, 0);
	SWSetSpriteDrawProc(gSimpleSpriteP, gSpriteDrawProc);
}


///--------------------------------------------------------------------------------------
// CreateDiamondMeterSprite
///--------------------------------------------------------------------------------------

void	CreateDiamondMeterSprite( void )
{
	OSErr				err;
	
		// Create the diamond meter sprite
	err = SWCreateSpriteFromPictResource(gSpriteWorldP, &gDiamondMeterSpriteP, NULL,
				kSpritePictResID, kSpritePictResID, 1, kPixelMask);
	FatalError(err);
	
	SWAddSprite(gSpriteLayerP, gDiamondMeterSpriteP);
	SWSetSpriteDrawProc(gDiamondMeterSpriteP, gSpriteDrawProc);
}


///--------------------------------------------------------------------------------------
// SetUpAnimation
///--------------------------------------------------------------------------------------

void	SetUpAnimation( void )
{
	Rect		moveBoundsRect;
	
	SWLockSpriteWorld(gSpriteWorldP);
	
		// Set up data used by the SmoothScrollingWorldMoveProc
	gScreenMidRect = gSimpleSpriteP->curFrameP->frameRect;
	CenterRect( &gScreenMidRect, &gSpriteWorldP->backRect );
	
	SWSetSpriteWorldMaxFPS(gSpriteWorldP, kMaxFPS);
	SWSyncSpriteWorldToVBL(gSpriteWorldP, kSyncToVBL);
	SWSetCleanUpSpriteWorld(gSpriteWorldP);
	
		// movement boundary = size of tileMap
	SetRect(&moveBoundsRect, 0, 0, 
		gSpriteWorldP->tileLayerArray[0]->numCols * gSpriteWorldP->tileWidth, 
		gSpriteWorldP->tileLayerArray[0]->numRows * gSpriteWorldP->tileHeight);
	
	SWSetScrollingWorldMoveBounds(gSpriteWorldP, &moveBoundsRect);
	SWSetScrollingWorldMoveProc(gSpriteWorldP, SmoothScrollingWorldMoveProc, gSimpleSpriteP);
	
	SWSetTileChangeProc(gSpriteWorldP, TileChangeProc);
	
		// Move visScrollRect to starting sprite position
	SWMoveVisScrollRect(gSpriteWorldP, 
		gSpriteWorldP->followSpriteP->destFrameRect.left - gSpriteWorldP->backRect.right/2,
		gSpriteWorldP->followSpriteP->destFrameRect.top - gSpriteWorldP->backRect.bottom/2);
	
		// Set starting position of diamond meter sprite
	DiamondMeterSpriteMoveProc(gDiamondMeterSpriteP);

	if (gSpriteWorldP->pixelDepth == 8)		// If in 256 colors
	{
		if (kInterlacedMode)
		{
			SWSetSpriteWorldScreenDrawProc(gSpriteWorldP, BP8BitInterlacedRectDrawProc);
			SWSetSpriteWorldOffscreenDrawProc(gSpriteWorldP, BP8BitInterlacedRectDrawProc);
			SWSetPartialMaskDrawProc(gSpriteWorldP, BP8BitInterlacedPartialMaskDrawProc);
			SWSetDoubleRectDrawProc(gSpriteWorldP, BP8BitInterlacedDoubleRectDrawProc);
		}
		else
		{
			SWSetSpriteWorldOffscreenDrawProc(gSpriteWorldP, BlitPixie8BitRectDrawProc);
			SWSetPartialMaskDrawProc(gSpriteWorldP, BlitPixie8BitPartialMaskDrawProc);
			SWSetDoubleRectDrawProc(gSpriteWorldP, BlitPixie8BitDoubleRectDrawProc);
			
#if !SW_PPC		// CopyBits is faster for full-screen blitting in PPC
			SWSetSpriteWorldScreenDrawProc(gSpriteWorldP, BlitPixie8BitRectDrawProc);
#endif
		}
	}	// When running on PPC, we can't use the AllBit blitters in lower than 8-bit
	else if ( !(SW_PPC && gSpriteWorldP->pixelDepth < 8) )	// Not 256 colors
	{
		if (kInterlacedMode && gSpriteWorldP->pixelDepth > 2)
		{
				// Use interlaced drawProcs unless in B&W, where interlacing is ugly
			SWSetSpriteWorldScreenDrawProc(gSpriteWorldP, BPAllBitInterlacedRectDrawProc);
			SWSetSpriteWorldOffscreenDrawProc(gSpriteWorldP, BPAllBitInterlacedRectDrawProc);
			SWSetPartialMaskDrawProc(gSpriteWorldP, BPAllBitInterlacedPartialMaskDrawProc);
			if (gSpriteWorldP->pixelDepth == 16)
				SWSetDoubleRectDrawProc(gSpriteWorldP, BP16BitInterlacedDoubleRectDrawProc);
		}
		else
		{
			SWSetSpriteWorldOffscreenDrawProc(gSpriteWorldP, BlitPixieAllBitRectDrawProc);
			SWSetSpriteWorldScreenDrawProc(gSpriteWorldP, BlitPixieAllBitRectDrawProc);
			SWSetPartialMaskDrawProc(gSpriteWorldP, BlitPixieAllBitPartialMaskDrawProc);
			if (gSpriteWorldP->pixelDepth == 16)
				SWSetDoubleRectDrawProc(gSpriteWorldP, BlitPixie16BitDoubleRectDrawProc);
		}
	}
	
	
		// Make sure CopyBits, if used, doesn't try to colorize things
	SWSetPortToWindow(gSpriteWorldP);
	ForeColor(blackColor);
	BackColor(whiteColor);
	
	SWDrawTilesInBackground(gSpriteWorldP);
	SWUpdateScrollingSpriteWorld(gSpriteWorldP, true);
}


///--------------------------------------------------------------------------------------
//  RunAnimation
///--------------------------------------------------------------------------------------

void	RunAnimation( void )
{
	unsigned long		frames;
	
	frames = 0;
	StartTimer();

	FatalError( SWStickyError() ); // Make sure no errors got past us during setup

	while (!Button())
	{
		SWProcessScrollingSpriteWorld(gSpriteWorldP);
		
			// We call this in case the Control Strip or something changes our window's visRgn
		KeepMenuBarHidden(gWindowP);

			// Move diamond meter sprite to new visScrollRect location
		DiamondMeterSpriteMoveProc(gDiamondMeterSpriteP);
		
		FatalError( SWStickyError() ); // Make sure no errors occurred during a MoveProc, etc.
	
		SWAnimateScrollingSpriteWorld(gSpriteWorldP);
		
		if (gSpriteWorldP->frameHasOccurred)
		{
			frames++;
			
			if (gSpriteHitExit)
				AdvanceLevel();
		}
	}
	
	
	SWShowMenuBar(gWindowP);
	ShowResults(frames);
}


///--------------------------------------------------------------------------------------
//  ShutDown (clean up and dispose of the SpriteWorld)
///--------------------------------------------------------------------------------------

void	ShutDown( void )
{
	SWDisposeSpriteWorld(&gSpriteWorldP);
	SWExitSpriteWorld();
	
	FlushEvents(everyEvent, 0);
	ShowCursor();
}


///--------------------------------------------------------------------------------------
//  TileChangeProc
///--------------------------------------------------------------------------------------

SW_FUNC void TileChangeProc(
	SpriteWorldPtr spriteWorldP)
{
	short			curImage;
	static short	diamondDelay = 0, exitDelay = 0;
	static short	oldTicks = 0;
	short			ticksPassed, ticks;
	
		// Initialize oldTicks the first time this function is called
	if (oldTicks == 0)
		oldTicks = TickCount();
	
	ticks = TickCount();
	ticksPassed = ticks - oldTicks;		// Number of ticks passed since last call
	oldTicks = ticks;
	
	
		// kDiamondTile
	diamondDelay += ticksPassed;
	if (diamondDelay >= kDiamondFrameRate)
	{
		curImage = spriteWorldP->curTileImage[kDiamondTile];
		if (curImage < kLastDiamondTile)
			curImage++;
		else
			curImage = kDiamondTile;
		
		SWChangeTileImage(spriteWorldP, kDiamondTile, curImage);
		diamondDelay = 0;
	}
	
	
		// Flash exit tile if all diamonds have been collected
	exitDelay += ticksPassed;
	if (exitDelay >= kExitFrameRate)
	{
		if  (!kRequireDiamondsToExit || gNumDiamonds == gNumDiamondsInMap)
		{
			curImage = spriteWorldP->curTileImage[ kFirstExitTile ];
			if ( curImage < kLastExitTile )
				curImage++;
			else
				curImage = kFirstExitTile;
			
			SWChangeTileImage( spriteWorldP, kFirstExitTile, curImage );
		}
		exitDelay = 0;
	}
}


///--------------------------------------------------------------------------------------
//  KeySpriteMoveProc
///--------------------------------------------------------------------------------------

SW_FUNC void KeySpriteMoveProc(SpritePtr srcSpriteP)
{
	short	row, col;
	short	rowDelta, colDelta;
	short	tile;
	
	UpdateKeys();	// Put the latest key values in the keys structure
	
	
	row = srcSpriteP->destFrameRect.top / kTileHeight;
	col = srcSpriteP->destFrameRect.left / kTileWidth;

	if (row * kTileHeight == srcSpriteP->destFrameRect.top &&
		col * kTileWidth == srcSpriteP->destFrameRect.left)
	{
		tile = gTileMap[row][col];
		
			// Leave black trail behind sprite
		if ( (tile >= kFirstGrassTile && tile <= kLastGrassTile) || tile == kDiamondTile)
			SWDrawTile(gSpriteWorldP, 0, row, col, kBlackTile);
			
		if (tile == kDiamondTile)
		{
			gNumDiamonds++;			// Increase number of diamonds sprite has collected
			UpdateDiamondMeter();	// Update meter
		}
		else if (tile == kFirstExitTile)
		{
			if  (!kRequireDiamondsToExit || gNumDiamonds == gNumDiamondsInMap)
			{
				gSpriteHitExit = true;
				return;
			}
		}
		

		rowDelta = 0;
		colDelta = 0;
		
		if (gKeys.left)
			colDelta = -1;
		else if (gKeys.right)
			colDelta = 1;
		else if (gKeys.up)
			rowDelta = -1;
		else if (gKeys.down)
			rowDelta = 1;
		
				
		
		tile = gTileMap[row + rowDelta][col + colDelta];
		
		if (tile < kFirstWallTile || tile > kLastWallTile)
		{
			srcSpriteP->vertMoveDelta = rowDelta * kSpriteMoveDelta;
			srcSpriteP->horizMoveDelta = colDelta * kSpriteMoveDelta;
		}
		else
		{
			srcSpriteP->vertMoveDelta = 0;
			srcSpriteP->horizMoveDelta = 0;
		}
	}
	
	SWOffsetSprite(srcSpriteP, srcSpriteP->horizMoveDelta, srcSpriteP->vertMoveDelta);
}


///--------------------------------------------------------------------------------------
//  DiamondMeterSpriteMoveProc - not installed as a MoveProc, but called directly
///--------------------------------------------------------------------------------------

SW_FUNC void DiamondMeterSpriteMoveProc(SpritePtr srcSpriteP)
{
		// Move sprite to top-left corner of current visScrollRect location
	SWMoveSprite(srcSpriteP, 
		gSpriteWorldP->visScrollRect.left + 10, 
		gSpriteWorldP->visScrollRect.top + 10);
}


///--------------------------------------------------------------------------------------
//  UpdateDiamondMeter - change the meter Sprite's image and mask
///--------------------------------------------------------------------------------------

SW_FUNC void UpdateDiamondMeter( void )
{
	double	percent;
	Rect	meterRect;
	
	percent = (double)gNumDiamonds / gNumDiamondsInMap;
	
		// 108 = length of meter; 17 = offset from start of meter
	SetRect(&meterRect, 17, 2, (108 * percent) + 17, 14);
	
	
		// Set port to our sprite's framePort GWorld
	SetGWorld(gDiamondMeterSpriteP->curFrameP->framePort, nil);
	
	ForeColor(magentaColor);
	PaintRect(&meterRect);
	
		// IMPORTANT: Set the color back when done! (In case CopyBits is used later)
	ForeColor(blackColor);
	
	
		// Set port to our sprite's pixel mask GWorld
	SetGWorld(gDiamondMeterSpriteP->curFrameP->maskPort, nil);
	
		// Mask image is inverted when in 8-bit or less
	if (gSpriteWorldP->pixelDepth <= 8)
		ForeColor(whiteColor);
	else
		ForeColor(blackColor);
	
	PaintRect(&meterRect);
	ForeColor(blackColor);
	
	
		// Set sprite to be redrawn, since we've changed its image
	gDiamondMeterSpriteP->needsToBeDrawn = true;
}


///--------------------------------------------------------------------------------------
//  SmoothScrollingWorldMoveProc - our scrolling WorldMoveProc
///--------------------------------------------------------------------------------------

SW_FUNC void SmoothScrollingWorldMoveProc(
	SpriteWorldPtr spriteWorldP,
	SpritePtr followSpriteP)
{	
	short	screenMidRectTop, screenMidRectLeft;
	
	screenMidRectTop = gScreenMidRect.top + spriteWorldP->visScrollRect.top;
	screenMidRectLeft = gScreenMidRect.left + spriteWorldP->visScrollRect.left;
	
	
	spriteWorldP->horizScrollDelta = (kSpriteMoveDelta * 
		(followSpriteP->destFrameRect.left - screenMidRectLeft) ) / kSpriteMoveDistance;
	
	spriteWorldP->vertScrollDelta = (kSpriteMoveDelta * 
		(followSpriteP->destFrameRect.top - screenMidRectTop) ) / kSpriteMoveDistance;
	
	if (kInterlacedMode)
		spriteWorldP->vertScrollDelta = spriteWorldP->vertScrollDelta>>1<<1;
}


///--------------------------------------------------------------------------------------
//  UpdateKeys (Put the latest key values in the keys structure)
///--------------------------------------------------------------------------------------

void	UpdateKeys( void )
{
	EventRecord		event;
	short			theKey;
	Boolean			isDown;
	
	while ( GetOSEvent( (keyUpMask | keyDownMask), &event ) )
	{
		theKey = (event.message & keyCodeMask) >> 8;
		isDown = (event.what != keyUp);
		
		if ( (theKey == kLeftArrowKey) || (theKey == kLeftKeyPad) )
			gKeys.left = isDown;
		else if ( (theKey == kRightArrowKey) || (theKey == kRightKeyPad) )
			gKeys.right = isDown;
		else if ( (theKey == kDownArrowKey) || (theKey == kDownKeyPad) )
			gKeys.down = isDown;
		else if ( (theKey == kUpArrowKey) || (theKey == kUpKeyPad) )
			gKeys.up = isDown;
	}
}


///--------------------------------------------------------------------------------------
//  AdvanceLevel
///--------------------------------------------------------------------------------------

void	AdvanceLevel( void )
{
	Rect			moveBoundsRect;
	unsigned long	junkTime;
	OSErr			err;
	
	gCurrentLevel++;
	gSpriteHitExit = false;
	
	SWDisposeTileMap(&gTileMapStructP);
	err = SWLoadTileMap(&gTileMapStructP, gCurrentLevel);
	if (err)
	{
		gCurrentLevel = 128;
		err = SWLoadTileMap(&gTileMapStructP, gCurrentLevel);
		FatalError(err);
	}
	
	gTileMap = gTileMapStructP->tileMap;
	SWInstallTileMap(gSpriteWorldP, gTileMapStructP, 0);
	
		// Reset scrolling moveBounds for new TileMap
	SetRect(&moveBoundsRect, 0, 0, 
		gTileMapStructP->numCols * kTileWidth, 
		gTileMapStructP->numRows * kTileHeight);
	SWSetScrollingWorldMoveBounds(gSpriteWorldP, &moveBoundsRect);
	
	Delay(15, &junkTime);
	
	CountDiamonds();
	
		// Update Diamond Meter Sprite, since we've changed its frame and mask
	SWUpdateSpriteFromPictResource(gDiamondMeterSpriteP, kSpritePictResID);
	SWUpdateFrameMasks(gSpriteWorldP, gDiamondMeterSpriteP->curFrameP);
	
	SWSetSpriteMoveDelta(gSimpleSpriteP, 0, 0);
	SWMoveSprite(gSimpleSpriteP, kStartCol * kTileWidth, kStartRow * kTileHeight);
	SWMoveVisScrollRect(gSpriteWorldP, 0, 0);
	
	SWResetCurrentTileImages(gSpriteWorldP);
	SWDrawTilesInBackground(gSpriteWorldP);
	SWUpdateSpriteWorld(gSpriteWorldP, false);
}
